@@ -14,16 +14,16 @@
  * limitations under the License.
  */
 
-#include "bson-compat.h"
+#include <bson/bson-compat.h>
 
 #include <limits.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "bson-context-private.h"
-#include "bson-oid.h"
-#include "bson-string.h"
+#include <bson/bson-context-private.h>
+#include <bson/bson-oid.h>
+#include <bson/bson-string.h>
 
 
 /*
@@ -32,7 +32,7 @@
  * to hex formatted ASCII. Performing two characters at a time roughly
  * reduces the number of operations by one-half.
  */
-static const uint16_t gHexCharPairs[] = {
+BSON_MAYBE_UNUSED static const uint16_t gHexCharPairs[] = {
 #if BSON_BYTE_ORDER == BSON_BIG_ENDIAN
    12336, 12337, 12338, 12339, 12340, 12341, 12342, 12343, 12344, 12345, 12385,
    12386, 12387, 12388, 12389, 12390, 12592, 12593, 12594, 12595, 12596, 12597,
