@@ -14,14 +14,12 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_READ_CONCERN_PRIVATE_H
 #define MONGOC_READ_CONCERN_PRIVATE_H
 
-#if !defined(MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
-
-#include <bson.h>
+#include <bson/bson.h>
 #include "mongoc-read-concern.h"
 
 
@@ -35,11 +33,14 @@ struct _mongoc_read_concern_t {
 };
 
 
-bool
-_mongoc_read_concern_is_default (const mongoc_read_concern_t *read_concern);
 const bson_t *
 _mongoc_read_concern_get_bson (mongoc_read_concern_t *read_concern);
 
+
+mongoc_read_concern_t *
+_mongoc_read_concern_new_from_iter (const bson_iter_t *iter,
+                                    bson_error_t *error);
+
 BSON_END_DECLS
 
 
