@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_SET_PRIVATE_H
 #define MONGOC_SET_PRIVATE_H
@@ -27,6 +27,7 @@ typedef void (*mongoc_set_item_dtor) (void *item, void *ctx);
 
 /* return true to continue iteration, false to stop */
 typedef bool (*mongoc_set_for_each_cb_t) (void *item, void *ctx);
+typedef bool (*mongoc_set_for_each_const_cb_t) (const void *item, void *ctx);
 typedef bool (*mongoc_set_for_each_with_id_cb_t) (uint32_t id,
                                                   void *item,
                                                   void *ctx);
@@ -56,11 +57,33 @@ mongoc_set_rm (mongoc_set_t *set, uint32_t id);
 void *
 mongoc_set_get (mongoc_set_t *set, uint32_t id);
 
-void *
-mongoc_set_get_item (mongoc_set_t *set, int idx);
+static BSON_INLINE const void *
+mongoc_set_get_const (const mongoc_set_t *set, uint32_t id)
+{
+   return mongoc_set_get ((mongoc_set_t *) set, id);
+}
 
 void *
-mongoc_set_get_item_and_id (mongoc_set_t *set, int idx, uint32_t *id /* OUT */);
+mongoc_set_get_item (mongoc_set_t *set, size_t idx);
+
+static BSON_INLINE const void *
+mongoc_set_get_item_const (const mongoc_set_t *set, size_t idx)
+{
+   return mongoc_set_get_item ((mongoc_set_t *) set, idx);
+}
+
+void *
+mongoc_set_get_item_and_id (mongoc_set_t *set,
+                            size_t idx,
+                            uint32_t *id /* OUT */);
+
+static BSON_INLINE const void *
+mongoc_set_get_item_and_id_const (const mongoc_set_t *set,
+                                  size_t idx,
+                                  uint32_t *id)
+{
+   return mongoc_set_get_item_and_id ((mongoc_set_t *) set, idx, id);
+}
 
 void
 mongoc_set_destroy (mongoc_set_t *set);
@@ -75,6 +98,15 @@ mongoc_set_destroy (mongoc_set_t *set);
 void
 mongoc_set_for_each (mongoc_set_t *set, mongoc_set_for_each_cb_t cb, void *ctx);
 
+static BSON_INLINE void
+mongoc_set_for_each_const (const mongoc_set_t *set,
+                           mongoc_set_for_each_const_cb_t cb,
+                           void *ctx)
+{
+   mongoc_set_for_each (
+      (mongoc_set_t *) set, (mongoc_set_for_each_cb_t) cb, ctx);
+}
+
 void
 mongoc_set_for_each_with_id (mongoc_set_t *set,
                              mongoc_set_for_each_with_id_cb_t cb,
@@ -88,7 +120,9 @@ mongoc_set_find_item (mongoc_set_t *set,
 
 /* id of first item in set for which "cb" returns true, or 0. */
 uint32_t
-mongoc_set_find_id (mongoc_set_t *set, mongoc_set_for_each_cb_t cb, void *ctx);
+mongoc_set_find_id (const mongoc_set_t *set,
+                    mongoc_set_for_each_const_cb_t cb,
+                    void *ctx);
 
 BSON_END_DECLS
 
